@@ -1,7 +1,7 @@
 // -*- C++ -*-
 //
 // This file is part of HepMC
-// Copyright (C) 2014-2020 The HepMC collaboration (see AUTHORS for details)
+// Copyright (C) 2014-2023 The HepMC collaboration (see AUTHORS for details)
 //
 #ifndef HEPMC3_FOURVECTOR_H
 #define HEPMC3_FOURVECTOR_H
@@ -43,12 +43,16 @@ public:
     FourVector(double xx, double yy, double zz, double ee)
         : m_v1(xx),     m_v2(yy),   m_v3(zz),      m_v4(ee)    {}
     /** @brief Copy constructor */
-    FourVector(const FourVector & v)
-        : m_v1(v.m_v1), m_v2(v.m_v2), m_v3(v.m_v3), m_v4(v.m_v4) {}
-
+    FourVector(const FourVector &) = default;
+    /** @brief Move constructor */
+    FourVector(FourVector && ) = default;
+    /** @brief = */
+    FourVector& operator=(const FourVector&) = default;
+    /** @brief = */
+    FourVector& operator=(FourVector&&) = default;
 
     /// @name Component accessors
-    //@{
+    /// @{
 
     /** @brief Set all FourVector fields, in order x,y,z,t */
     void set(double x1, double x2, double x3, double x4) {
@@ -134,16 +138,18 @@ public:
     /// @deprecated Prefer the HepMC-style set_y() function
     void setE(double ee) { set_e(ee);    }
 
-    //@}
+    /// @}
 
 
     /// @name Computed properties
-    //@{
+    /// @{
 
     /// Squared magnitude of (x, y, z) 3-vector
     double length2()  const { return x()*x() + y()*y() + z()*z(); }
     /// Magnitude of spatial (x, y, z) 3-vector
     double length()  const { return std::sqrt(length2()); }
+    /// Magnitude of spatial (x, y, z) 3-vector, for HepMC2 compatibility
+    double rho()    const { return length(); }
     /// Squared magnitude of (x, y) vector
     double perp2()      const { return  x()*x() + y()*y(); }
     /// Magnitude of (x, y) vector
@@ -169,23 +175,32 @@ public:
     /// Polar angle w.r.t. z direction
     double theta() const {  return std::atan2( perp(), z() ); }
     /// Pseudorapidity
-    double eta() const  { return ( p3mod() == 0.0 ) ? 0.0: (0.5*std::log( (p3mod() + pz()) / (p3mod() - pz()) )); }
+    double eta() const {
+      if ( p3mod() == 0.0 )  return 0.0;
+      if ( p3mod() == fabs(pz()) ) return std::copysign(HUGE_VAL, pz());
+      return 0.5*std::log( (p3mod() + pz()) / (p3mod() - pz()) );
+    }
     /// Rapidity
-    double rap() const {   return ( e() == 0.0 ) ? 0.0: (0.5*std::log( (e() + pz()) / (e() - pz()) )); }
+    double rap() const {
+      if ( e() == 0.0 ) return 0.0;
+      if ( e() == fabs(pz()) ) return std::copysign(HUGE_VAL, pz());
+      return 0.5*std::log( (e() + pz()) / (e() - pz()) );
+    }
     /// Absolute pseudorapidity
     double abs_eta() const { return std::abs( eta() ); }
     /// Absolute rapidity
     double abs_rap() const { return std::abs( rap() ); }
 
     /// Same as eta()
+    ///
     /// @deprecated Prefer 'only one way to do it', and we don't have equivalent long names for e.g. pid, phi or eta
     double pseudoRapidity() const { return eta(); }
 
-    //@}
+    /// @}
 
 
     /// @name Comparisons to another FourVector
-    //@{
+    /// @{
 
     /// Check if the length of this vertex is zero
     bool is_zero() const { return x() == 0 && y() == 0 && z() == 0 && t() == 0; }
@@ -225,11 +240,11 @@ public:
         return std::sqrt( delta_r2_rap(v) );
     }
 
-    //@}
+    /// @}
 
 
     /// @name Operators
-    //@{
+    /// @{
 
     /// Equality
     bool operator==(const FourVector& rhs) const {
@@ -284,7 +299,7 @@ public:
         setT(t()/rhs);
     }
 
-    //@}
+    /// @}
 
 
     /// Static null FourVector = (0,0,0,0)
@@ -305,7 +320,7 @@ private:
 
 
 /// @name Unbound vector comparison functions
-//@{
+/// @{
 
 /// Signed azimuthal angle separation in [-pi, pi] between vecs @c a and @c b
 inline double delta_phi(const FourVector &a, const FourVector &b) { return b.delta_phi(a); }
@@ -328,7 +343,7 @@ inline double delta_r2_rap(const FourVector &a, const FourVector &b) { return b.
 /// R_rap-distance separation dR = sqrt(dphi^2 + drap^2) between vecs @c a and @c b
 inline double delta_r_rap(const FourVector &a, const FourVector &b) { return b.delta_r_rap(a); }
 
-//@}
+/// @}
 
 
 } // namespace HepMC3
